/*
 * vkrunner
 *
 * Copyright (C) 2018 Intel Corporation
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the next
 * paragraph) shall be included in all copies or substantial portions of the
 * Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#ifndef VR_PIPELINE_H
#define VR_PIPELINE_H

#include "vr-script-private.h"
#include "vr-window.h"
#include "vr-config.h"
#include "vr-pipeline-key.h"

struct vr_pipeline {
        struct vr_window *window;
        VkPipelineLayout layout;
        VkDescriptorPool descriptor_pool;
        VkDescriptorSetLayout *descriptor_set_layout;
        unsigned n_desc_sets;
        int n_pipelines;
        VkPipeline *pipelines;
        VkPipelineCache pipeline_cache;
        VkShaderModule modules[VR_SHADER_STAGE_N_STAGES];
        VkShaderStageFlagBits stages;
};

struct vr_pipeline_vertex {
        float x, y, z;
};

struct vr_pipeline *
vr_pipeline_create(const struct vr_config *config,
                   struct vr_window *window,
                   const struct vr_script *script);

void
vr_pipeline_free(struct vr_pipeline *pipeline);

#endif /* VR_PIPELINE_H */
